# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations

import re
from datetime import datetime
from datetime import timedelta
from datetime import timezone
from enum import Enum
from typing import Any

# Based on https://cwiki.apache.org/confluence/display/TIKA/Metadata+Overview

_TIME_RE = re.compile(
    r"(?P<year>\d{4})-"
    r"(?P<month>\d{2})-"
    r"(?P<day>\d{2})"
    r"[ tT]"
    r"(?P<hour>\d{2}):"
    r"(?P<minute>\d{2}):"
    r"(?P<second>\d{2})"
    r"(?P<fractional_seconds>\.\d+)?"
    r"(?P<timezone>[zZ]|[+-]\d{2}:\d{2})?",
)


class TikaKey(str, Enum):
    """
    Keys for access to certain Tika returned values in the JSON.

    Based on
      - https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=235835139#MetadataOverview-TikaProcess
      - https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=235835139#MetadataOverview-TikaGeneral
    """

    Parsers = "X-TIKA:Parsed-By"
    Parser_Full = "X-TIKA:Parsed-By-Full-Set"
    Parse_Time = "X-TIKA:parse_time_millis"
    ContentType = "Content-Type"
    ContentLength = "Content-Length"
    Content = "X-TIKA:content"


class DublinCoreKey(str, Enum):
    """
    Dublin Core keys for access to certain Tika returned values in the JSON.

    Based on:
      - https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=235835139#MetadataOverview-DublinCore
    """

    Creator = "dc:creator"
    Created = "dcterms:created"
    Modified = "dcterms:modified"
    Rights = "dc:rights"
    Contributor = "dc:contributor"
    Title = "dc:title"
    Relation = "dc:relation"
    Type = "dc:type"
    Identifier = "dc:identifier"
    Publisher = "dc:publisher"
    Description = "dc:description"
    Subject = "dc:subject"
    Language = "dc:language"
    Format = "dc:format"


class XmpKey(str, Enum):
    """
    XMP keys for access to certain Tika returned values in the JSON.

    Based on:
      - https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=235835139#MetadataOverview-XMP(eXtensibleMetadataPlatform)
    """

    About = "xmp:About"
    Created = "xmp:CreateDate"
    NumPages = "xmpTPg:NPages"


class OtherTikaKeys(str, Enum):
    """Other keys Tika may return in the JSON."""

    CharacterCount = "meta:character-count"
    LastAuthor = "meta:last-author"
    Revision = "cp:revision"
    Language = "language"


class TikaResponse:
    """
    A basic wrapper class for the JSON data returned from the Tika server.

    It sets fields which the response always appears to have, and some small helpers for getting and converting
    other data types, including handling the chance those don't exist in the response.

    All returned data is available in the decoded JSON form under the .data attribute
    """

    def __init__(self, data: dict[str | TikaKey | DublinCoreKey | XmpKey | OtherTikaKeys, Any]) -> None:
        """Construct a TikaResponse using the provided JSON data from a Tika server."""
        self.data = data

        # Always set keys
        self.type: str = self.data[TikaKey.ContentType]
        self.parsers: list[str] = self.data[TikaKey.Parsers]

        # Tika keys
        self.content: str | None = data.get(TikaKey.Content)
        self.content_length: int | None = int(self.data.get(TikaKey.ContentLength, "0")) or None

        # Dublin Core keys
        self.created: datetime | None = self.parse_datetime_string(self.data.get(DublinCoreKey.Created))
        self.modified: datetime | None = self.parse_datetime_string(self.data.get(DublinCoreKey.Modified))
        self.title: str | None = self.data.get(DublinCoreKey.Title)

        # Xmp keys
        self.xmp_created: datetime | None = self.parse_datetime_string(self.data.get(XmpKey.Created))
        self.page_count: int | None = int(self.data.get(XmpKey.NumPages, "0")) or None

        # Other general keys
        self.character_count: int | None = int(self.data.get(OtherTikaKeys.CharacterCount, "0")) or None
        self.revision: int | None = int(self.data.get(OtherTikaKeys.Revision, "0")) or None
        self.language: str | None = self.data.get(OtherTikaKeys.Language)
        self.last_author: str | None = self.data.get(OtherTikaKeys.LastAuthor)

    @staticmethod
    def parse_datetime_string(
        date_str: str | None,
    ) -> datetime | None:
        """
        If present, attempts to parse the given key as an ISO-8061 format datetime, including timezone handling.

        If not present, return None
        """
        if not date_str:
            return None

        m = _TIME_RE.match(date_str)
        if not m:
            return None

        (year, month, day, hour, minute, second, frac_sec, timezone_str) = m.groups()

        # Parse fractional seconds without float conversion to avoid precision loss
        if frac_sec is not None:
            # Remove the leading dot and pad/truncate to 6 digits
            frac_str = frac_sec[1:]  # Remove the '.'
            frac_str = frac_str.ljust(6, "0")[:6]  # Pad with zeros or truncate to 6 digits
            microseconds = int(frac_str)
        else:
            microseconds = 0

        tzinfo = None
        if timezone_str is not None:
            if timezone_str.lower() == "z":
                tzinfo = timezone.utc
            else:
                multi = -1 if timezone_str[0:1] == "-" else 1
                hours = int(timezone_str[1:3])
                minutes = int(timezone_str[4:])
                delta = timedelta(hours=hours, minutes=minutes) * multi
                tzinfo = timezone(delta)

        return datetime(
            year=int(year),
            month=int(month),
            day=int(day),
            hour=int(hour),
            minute=int(minute),
            second=int(second),
            microsecond=microseconds,
            tzinfo=tzinfo,
        )

    def __repr__(self) -> str:  # pragma: no cover
        """Representation of this class."""
        return f"{self.type} response"



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations

from typing import TYPE_CHECKING
from typing import Final

from tika_client._base import AsyncResource
from tika_client._base import SyncResource

PLAIN_TEXT_ENDPOINT: Final[str] = "/tika/text"
PLAIN_TEXT_MULTI_PART_ENDPOINT: Final[str] = "/tika/form/text"
HTML_ENDPOINT: Final[str] = "/tika"
HTML_MULTI_PART_ENDPOINT: Final[str] = "/tika/form"

if TYPE_CHECKING:
    from pathlib import Path

    from httpx import AsyncClient
    from httpx import Client

    from tika_client.data_models import TikaResponse


class SyncTikaHtml(SyncResource):
    def from_file(self, filepath: Path, mime_type: str | None = None) -> TikaResponse:
        """
        Return the formatted (as HTML) document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(self.put_multipart(HTML_MULTI_PART_ENDPOINT, filepath, mime_type))

    def from_buffer(self, content: str | bytes, mime_type: str | None = None) -> TikaResponse:
        """
        Return the HTML formatted document data from a given string of document content.

        Args:
            content: The content to be sent to the Tika server
            mime_type: The mime type of the content to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(self.put_content(HTML_ENDPOINT, content, mime_type))


class SyncTikaPlain(SyncResource):
    def from_file(self, filepath: Path, mime_type: str | None = None) -> TikaResponse:
        """
        Return the plain text document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(self.put_multipart(PLAIN_TEXT_MULTI_PART_ENDPOINT, filepath, mime_type))

    def from_buffer(self, content: str | bytes, mime_type: str | None = None) -> TikaResponse:
        """
        Return the plain text document data from a given string of document content.

        Args:
            content: The content to be sent to the Tika server
            mime_type: The mime type of the content to be sent to the Tika server

        Returns:
            The JSON response from the

        """
        return self.decoded_response(self.put_content(PLAIN_TEXT_ENDPOINT, content, mime_type))


class SyncTika(SyncResource):
    """
    Handles interaction with the /tika endpoint of a Tika server REST API.

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-TikaResource
    """

    def __init__(self, client: Client, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        self.as_html = SyncTikaHtml(self.client, compress=compress)
        self.as_text = SyncTikaPlain(self.client, compress=compress)


class AsyncTikaHtml(AsyncResource):
    async def from_file(self, filepath: Path, mime_type: str | None = None) -> TikaResponse:
        """
        Return the formatted (as HTML) document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(await self.put_multipart(HTML_MULTI_PART_ENDPOINT, filepath, mime_type))

    async def from_buffer(self, content: str | bytes, mime_type: str | None = None) -> TikaResponse:
        """
        Return the HTML formatted document data from a given string of document content.

        Args:
            content: The content to be sent to the Tika server
            mime_type: The mime type of the content to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(await self.put_content(HTML_ENDPOINT, content, mime_type))


class AsyncTikaPlain(AsyncResource):
    async def from_file(self, filepath: Path, mime_type: str | None = None) -> TikaResponse:
        """
        Return the plain text document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            The JSON response from the Tika server

        """
        return self.decoded_response(await self.put_multipart(PLAIN_TEXT_MULTI_PART_ENDPOINT, filepath, mime_type))

    async def from_buffer(self, content: str | bytes, mime_type: str | None = None) -> TikaResponse:
        """
        Return the plain text document data from a given string of document content.

        Args:
            content: The content to be sent to the Tika server
            mime_type: The mime type of the content to be sent to the Tika server

        Returns:
            The JSON response from the

        """
        return self.decoded_response(await self.put_content(PLAIN_TEXT_ENDPOINT, content, mime_type))


class AsyncTika(AsyncResource):
    """
    Handles interaction with the /tika endpoint of a Tika server REST API.

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-TikaResource
    """

    def __init__(self, client: AsyncClient, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        self.as_html = AsyncTikaHtml(self.client, compress=compress)
        self.as_text = AsyncTikaPlain(self.client, compress=compress)



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0

from __future__ import annotations

from typing import TYPE_CHECKING
from typing import Final

from tika_client._base import AsyncResource
from tika_client._base import SyncResource

if TYPE_CHECKING:
    from pathlib import Path

    from httpx import AsyncClient
    from httpx import Client

    from tika_client.data_models import TikaResponse

HTML_ENDPOINT: Final[str] = "/rmeta"
HTML_MULTI_PART_ENDPOINT: Final[str] = "/rmeta/form/html"
PLAIN_TEXT_ENDPOINT: Final[str] = "/rmeta/text"
PLAIN_TEXT_MULTI_PART_ENDPOINT: Final[str] = "/rmeta/form/text"


class SyncTikaRmetaBase(SyncResource):
    def common_call(
        self,
        endpoint: str,
        filepath: Path,
        mime_type: str | None = None,
    ) -> list[TikaResponse]:
        """
        Given a specific endpoint and a file, do a multipart put to the endpoint.

        Args:
            endpoint: The endpoint to send the file to
            filepath: The path to the file to send
            mime_type: The mime type of the file to send

        Returns:
            A list of JSON responses from the Tika server

        """
        return [self.decoded_response(item) for item in self.put_multipart(endpoint, filepath, mime_type)]


class SyncRecursiveMetaHtml(SyncTikaRmetaBase):
    def from_file(self, filepath: Path, mime_type: str | None = None) -> list[TikaResponse]:
        """
        Return the formatted (as HTML) document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            A list of JSON responses from the Tika server

        """
        return self.common_call(HTML_MULTI_PART_ENDPOINT, filepath, mime_type)


class SyncRecursiveMetaPlain(SyncTikaRmetaBase):
    def from_file(self, filepath: Path, mime_type: str | None = None) -> list[TikaResponse]:
        """
        Return the plain text document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            A list of JSON responses from the Tika server

        """
        return self.common_call(PLAIN_TEXT_MULTI_PART_ENDPOINT, filepath, mime_type)


class SyncRecursive(SyncResource):
    """
    Handles interaction with the /rmeta endpoint of a Tika server REST API.

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-RecursiveMetadataandContent
    """

    def __init__(self, client: Client, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        # No support for XML endpoint.  Who wants that?
        self.as_html = SyncRecursiveMetaHtml(self.client, compress=compress)
        self.as_text = SyncRecursiveMetaPlain(self.client, compress=compress)


class AsyncTikaRmetaBase(AsyncResource):
    async def common_call(
        self,
        endpoint: str,
        filepath: Path,
        mime_type: str | None = None,
    ) -> list[TikaResponse]:
        """
        Given a specific endpoint and a file, do a multipart put to the endpoint.

        Args:
            endpoint: The endpoint to send the file to
            filepath: The path to the file to send
            mime_type: The mime type of the file to send

        Returns:
            A list of JSON responses from the Tika server

        """
        return [self.decoded_response(item) for item in await self.put_multipart(endpoint, filepath, mime_type)]


class AsyncRecursiveMetaHtml(AsyncTikaRmetaBase):
    async def from_file(self, filepath: Path, mime_type: str | None = None) -> list[TikaResponse]:
        """
        Return the formatted (as HTML) document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            A list of JSON responses from the Tika server

        """
        return await self.common_call(HTML_MULTI_PART_ENDPOINT, filepath, mime_type)


class AsyncRecursiveMetaPlain(AsyncTikaRmetaBase):
    async def from_file(self, filepath: Path, mime_type: str | None = None) -> list[TikaResponse]:
        """
        Return the plain text document data.

        Args:
            filepath: The path to the file to be sent to the Tika server
            mime_type: The mime type of the file to be sent to the Tika server

        Returns:
            A list of JSON responses from the Tika server

        """
        return await self.common_call(PLAIN_TEXT_MULTI_PART_ENDPOINT, filepath, mime_type)


class AsyncRecursive(AsyncResource):
    """
    Handles interaction with the /rmeta endpoint of a Tika server REST API.

    See documentation:
    https://cwiki.apache.org/confluence/display/TIKA/TikaServer#TikaServer-RecursiveMetadataandContent
    """

    def __init__(self, client: AsyncClient, *, compress: bool) -> None:
        super().__init__(client, compress=compress)
        # No support for XML endpoint.  Who wants that?
        self.as_html = AsyncRecursiveMetaHtml(self.client, compress=compress)
        self.as_text = AsyncRecursiveMetaPlain(self.client, compress=compress)



from pathlib import Path

import magic

from tika_client.client import AsyncTikaClient
from tika_client.client import TikaClient


class TestParseImageMetadata:
    def test_image_jpeg(self, tika_client: TikaClient, sample_jpeg_file: Path) -> None:
        """
        Test the handling of a JPEG file metadata retrieval
        """
        resp = tika_client.metadata.from_file(sample_jpeg_file, magic.from_file(str(sample_jpeg_file), mime=True))

        assert resp.type == "image/jpeg"

    def test_image_png(self, tika_client: TikaClient, sample_png_file: Path) -> None:
        """
        Test the handling of a PNG file metadata retrieval
        """
        resp = tika_client.metadata.from_file(sample_png_file, magic.from_file(str(sample_png_file), mime=True))

        assert resp.type == "image/png"


class TestAsyncParseImageMetadata:
    async def test_image_jpeg(
        self,
        async_tika_client: AsyncTikaClient,
        sample_jpeg_file: Path,
    ) -> None:
        """
        Test the handling of a JPEG file metadata retrieval using async client
        """
        resp = await async_tika_client.metadata.from_file(
            sample_jpeg_file,
            magic.from_file(str(sample_jpeg_file), mime=True),
        )

        assert resp.type == "image/jpeg"

    async def test_image_png(
        self,
        async_tika_client: AsyncTikaClient,
        sample_png_file: Path,
    ) -> None:
        """
        Test the handling of a PNG file metadata retrieval using async client
        """
        resp = await async_tika_client.metadata.from_file(
            sample_png_file,
            magic.from_file(str(sample_png_file), mime=True),
        )

        assert resp.type == "image/png"
